@@ -102,7 +102,7 @@ bool App_WithDoc::open_document(const Glib::ustring& file_uri)
     }
 
     //Tell user that it's already open:
-    ui_warning(gettext("This document is already open."));
+    ui_warning(gettext("Document already open"), gettext("This document is already open."));
 
     return true; //success.
   }
@@ -129,9 +129,30 @@ bool App_WithDoc::open_document(const Glib::ustring& file_uri)
     pApp->m_pDocument->set_file_uri(file_uri);
     bool bTest = pApp->m_pDocument->load();
 
+    bool bOpenFailed = false;
     if(!bTest) //if open failed.
+      bOpenFailed = true;
+    else
     {
-      ui_warning(gettext("Open failed."));
+      //if open succeeded then let the App respond:
+      bool test = pApp->on_document_load();
+      if(!test)
+        bOpenFailed = true; //The application didn't like something about the just-loaded document.
+      else
+      {
+        pApp->update_window_title();
+        set_document_modified(false); //disables menu and toolbar Save items.
+
+        //Update document history list:
+        document_history_add(file_uri);
+
+        return true; //success.
+      }
+    }
+
+    if(bOpenFailed)
+    {
+      ui_warning(gettext("Open failed."), gettext("The document could not be opened."));
 
       if(bUsingNewInstance)
       {
@@ -149,18 +170,7 @@ bool App_WithDoc::open_document(const Glib::ustring& file_uri)
 
       return false; //failed.
     }
-    else
-    {
-      //if open succeeded:
-      pApp->on_document_load();
-      pApp->update_window_title();
-      set_document_modified(false); //disables menu and toolbar Save items.
-
-      //Update document history list:
-      document_history_add(file_uri);
-
-      return true; //success.
-    }
+      
   } //if already open.
 
   return false; //failed.
@@ -185,44 +195,51 @@ void App_WithDoc::offer_saveas()
   on_menu_file_saveas();
 }
 
+bool App_WithDoc::file_exists(const Glib::ustring& uri)
+{
+  //Check whether file exists already:
+  {
+    // Try to open the input file.
+    Gnome::Vfs::Handle read_handle;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    try
+    {
+      read_handle.open(uri, Gnome::Vfs::OPEN_READ);
+
+      //It does (there was no exception):
+      return true;
+    }
+    catch(const Gnome::Vfs::exception& /* ex */)
+    {
+      return false; //It does not exist.
+    }
+#else
+      std::auto_ptr<Gnome::Vfs::exception> error;
+      read_handle.open(uri, Gnome::Vfs::OPEN_READ, error);
+      if(error.get() != NULL) return false;
+#endif
+  }
+}
+
 void App_WithDoc::on_menu_file_saveas()
 {
   //Display File Save dialog and respond to choice:
-   
+
   //Bring document window to front, to make it clear which document is being saved:
   //This doesn't work: TODO.
   ui_bring_to_front();
 
   //Show the save dialog:  
   const Glib::ustring& file_uriOld = m_pDocument->get_file_uri();
-  Glib::ustring file_uri = ui_file_select_save(file_uriOld);
+
+  Glib::ustring file_uri = ui_file_select_save(file_uriOld); //Also asks for overwrite confirmation.
   if(!file_uri.empty())
   {
     //Enforce the file extension:
     file_uri = m_pDocument->get_file_uri_with_extension(file_uri);
-    
+
     bool bUseThisFileUri = true;
 
-    //Check whether file exists already:
-    {
-      // Try to open the input file.
-      Gnome::Vfs::Handle read_handle;
-      try
-      {
-        read_handle.open(file_uri, Gnome::Vfs::OPEN_READ);
-
-        //It does (there was no exception), so ask the user to confirm overwrite:
-        bool bOverwrite = ui_ask_overwrite(file_uri);
-
-        //Respond to button that was clicked:
-        bUseThisFileUri = bOverwrite;
-      }
-      catch(const Gnome::Vfs::exception& ex)
-      {
-        bUseThisFileUri = true; //It does not exist.
-      }
-    }
-
     //Save to this filepath:
     if(bUseThisFileUri)
     {
@@ -231,7 +248,7 @@ void App_WithDoc::on_menu_file_saveas()
 
       if(!bTest)
       {
-        ui_warning(gettext("Save failed."));
+        ui_warning(gettext("Save failed."), gettext("There was an error while saving the file. Your changes have not been saved."));
       }
       else
       {
@@ -273,8 +290,8 @@ void App_WithDoc::on_menu_file_save()
       if(bTest)
       {
         //Disable Save and SaveAs menu items:
-        after_successful_save();    
-   
+        after_successful_save();
+
         //Close the document if this save was in response to a 'Do you want to save before closing?':
         //if(m_bCloseAfterSave) // || m_bExiting
         //  close_mark_or_destroy();
@@ -282,8 +299,8 @@ void App_WithDoc::on_menu_file_save()
       else
       {
         //The save failed. Tell the user and don't do anything else:
-        ui_warning(gettext("Save failed."));
-        
+        ui_warning(gettext("Save failed."), gettext("There was an error while saving the file. Your changes have not been saved."));
+
         cancel_close_or_exit();
       }
     }
@@ -396,9 +413,9 @@ void App_WithDoc::cancel_close_or_exit()
 
   //exit_destroy_marked_instances(); //Clean up after an exit.
 }
-  
-void App_WithDoc::on_document_load()
-{        
+
+bool App_WithDoc::on_document_load()
+{
   //Show document contents:
   if(m_pDocument)
   {
@@ -408,7 +425,11 @@ void App_WithDoc::on_document_load()
 
     //Set document as unmodified (this could have been wrongly set during the load):
     set_document_modified(false);
+
+    return true;
   }
+  else
+    return false; //I can't think of any reason why this would happen.
   
   //If you are not using Views, then override this to fill your various windows with stuff according to the contents of the document.
 }
