@@ -19,12 +19,22 @@
 #ifndef DIALOG_OVERWRITEFILE_H
 #define DIALOG_OVERWRITEFILE_H
 
+#include <bakeryconfig.h>
+
+#ifdef BAKERY_MAEMO_ENABLED
+#include <hildonmm/note.h>
+#else
 #include <gtkmm/messagedialog.h>
+#endif
 
 namespace Bakery
 {
 
+#ifdef BAKERY_MAEMO_ENABLED
+class Dialog_OverwriteFile : public Hildon::Note
+#else
 class Dialog_OverwriteFile : public Gtk::MessageDialog
+#endif
 {
 public:
   Dialog_OverwriteFile(const Glib::ustring& filepath);
